@@ -7,8 +7,9 @@ from django.contrib import messages
 from django.contrib.auth.decorators import login_required
 from django.contrib.sites.models import RequestSite
 from django.contrib.sites.models import Site
-from django.contrib.auth.models import BaseUserManager
+from django.contrib.auth.models import BaseUserManager, Group
 from django.conf import settings
+from django.db.models import FieldDoesNotExist
 
 
 from registration.views import RegistrationView as BaseRegistrationView
@@ -32,15 +33,32 @@ class RegistrationView(BaseRegistrationView):
         else:
             site = RequestSite(request)
         user_fields = {}
-        # keep non password fields
         for field in compat.get_registration_fields():
-            if not field.startswith('password'):
-                user_fields[field] = cleaned_data[field]
+            # save User model fields
+            try:
+                User._meta.get_field(field)
+            except FieldDoesNotExist:
+                continue
+            if field.startswith('password'):
+                continue
+            user_fields[field] = cleaned_data[field]
             if field == 'email':
                 user_fields[field] = BaseUserManager.normalize_email(user_fields[field])
         new_user = User(is_active=False, **user_fields)
+        new_user.clean()
         new_user.set_password(cleaned_data['password1'])
         new_user.save()
+        attributes = models.Attribute.objects.filter(
+                asked_on_registration=True)
+        if attributes:
+            for attribute in attributes:
+                attribute.set_value(new_user, cleaned_data[attribute.name])
+        if app_settings.A2_REGISTRATION_GROUPS:
+            groups = []
+            for name in app_settings.A2_REGISTRATION_GROUPS:
+                group, created = Group.objects.get_or_create(name=name)
+                groups.append(group)
+            new_user.groups = groups
         registration_profile = RegistrationProfile.objects.create_profile(new_user)
         registration_profile.send_activation_email(site)
 
