@@ -8,6 +8,10 @@ from django.conf import settings
 from django.forms import ModelForm
 import django.forms
 from django.contrib import messages
+try:
+    from django.contrib.contenttypes.admin import GenericTabularInline
+except ImportError:
+    from django.contrib.contenttypes.generic import GenericTabularInline
 
 from authentic2.saml.models import LibertyProvider, LibertyServiceProvider
 from authentic2.saml.models import LibertyIdentityProvider, IdPOptionsSPPolicy
@@ -16,8 +20,12 @@ from authentic2.saml.models import LibertyProviderPolicy
 from authentic2.saml.models import LibertySessionDump, LibertyFederation
 from authentic2.saml.models import LibertyAssertion, LibertySessionSP, KeyValue
 from authentic2.saml.models import LibertySession
+from authentic2.saml.models import SAMLAttribute
 from authentic2.http_utils import get_url
 
+from authentic2.decorators import to_iter
+from authentic2.attributes_ng.engine import get_attribute_names
+
 from . import admin_views
 
 logger = logging.getLogger(__name__)
@@ -157,6 +165,15 @@ def update_metadata(modeladmin, request, queryset):
         messages.info(request, _('Metadata update for: %s') % updated)
 
 
+class SAMLAttributeInlineAdmin(GenericTabularInline):
+    model = SAMLAttribute
+
+    def formfield_for_choice_field(self, db_field, request, **kwargs):
+        if db_field.name == 'attribute_name':
+            kwargs['choices'] = to_iter(get_attribute_names)({'user': None, 'request': None})
+        return super(SAMLAttributeInlineAdmin, self).formfield_for_choice_field(db_field, request, **kwargs)
+
+
 class LibertyProviderAdmin(admin.ModelAdmin):
     form = LibertyProviderForm
     list_display = ('name', 'slug', 'entity_id', 'protocol_conformance')
@@ -174,7 +191,8 @@ class LibertyProviderAdmin(admin.ModelAdmin):
     )
     inlines = [
             LibertyServiceProviderInline,
-            LibertyIdentityProviderInline
+            LibertyIdentityProviderInline,
+            SAMLAttributeInlineAdmin,
     ]
     actions = [ update_metadata ]
     prepopulated_fields = {'slug': ('name',)}
@@ -184,7 +202,7 @@ class LibertyProviderAdmin(admin.ModelAdmin):
         urls = patterns('',
             url(r'^add-from-url/$',
                 self.admin_site.admin_view(admin_views.AddLibertyProviderFromUrlView.as_view(model_admin=self)),
-                name='eo_facture_facture_add_from_url'),
+                name='saml_libertyprovider_add_from_url'),
             ) + urls
         return urls
 
@@ -194,7 +212,6 @@ class LibertyProviderPolicyAdmin(admin.ModelAdmin):
             LibertyServiceProviderInline,
     ]
 
-
 class LibertyFederationAdmin(admin.ModelAdmin):
     search_fields = ('name_id_content', 'user__username')
     list_display = ('user', 'creation', 'last_modification', 'name_id_content', 'format', 'idp', 'sp')
@@ -207,7 +224,29 @@ class LibertyFederationAdmin(admin.ModelAdmin):
         return name_id_format
 
 admin.site.register(IdPOptionsSPPolicy, IdPOptionsSPPolicyAdmin)
-admin.site.register(SPOptionsIdPPolicy)
+
+class SPOptionsIdPPolicyAdmin(admin.ModelAdmin):
+    inlines = [ SAMLAttributeInlineAdmin ]
+    fields = (
+            'name',
+            'enabled',
+            'prefered_assertion_consumer_binding',
+            'encrypt_nameid',
+            'encrypt_assertion',
+            'authn_request_signed',
+            'idp_initiated_sso',
+            'default_name_id_format',
+            'accepted_name_id_format',
+            'ask_user_consent',
+            'accept_slo',
+            'forward_slo',
+            'needs_iframe_logout',
+            'iframe_logout_timeout',
+            'http_method_for_slo_request',
+    )
+
+
+admin.site.register(SPOptionsIdPPolicy, SPOptionsIdPPolicyAdmin)
 admin.site.register(LibertyProvider, LibertyProviderAdmin)
 admin.site.register(LibertyProviderPolicy, LibertyProviderPolicyAdmin)
 
