@@ -1,22 +1,30 @@
 import xml.etree.ElementTree as etree
 import hashlib
-import binascii
-import base64
+import numbers
 import datetime
 
 import lasso
 from django.db import models
+from django.db.models import Q
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.utils.translation import ugettext_lazy as _
 from django.core.exceptions import ObjectDoesNotExist
-from django.utils.importlib import import_module
-from django.utils.timezone import now
-from django.dispatch import Signal
+from django.contrib.contenttypes.models import ContentType
+try:
+    from django.contrib.contenttypes.fields import GenericForeignKey
+except ImportError:
+    from django.contrib.contenttypes.generic import GenericForeignKey
+try:
+    from django.contrib.contenttypes.fields import GenericRelation
+except ImportError:
+    from django.contrib.contenttypes.generic import GenericRelation
+
 
 from fields import PickledObjectField, MultiSelectField
 
-from . import app_settings
+from . import app_settings, managers
+from .. import managers as a2_managers
 
 def metadata_validator(meta):
     provider=lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, meta.encode('utf8'))
@@ -295,9 +303,13 @@ class SPOptionsIdPPolicy(models.Model):
                 'onload event for this iframe and the moment we consider its '
                 'loading to be really finished'),
             default=300)
+    http_method_for_slo_request = models.IntegerField(
+            verbose_name = _("HTTP binding for the SLO requests"),
+            choices = HTTP_METHOD, default = lasso.HTTP_METHOD_REDIRECT)
     federation_mode = models.PositiveIntegerField(_('federation mode'),
             choices=app_settings.FEDERATION_MODE.get_choices(app_settings),
             default=app_settings.FEDERATION_MODE.get_default(app_settings))
+    attributes = GenericRelation('SAMLAttribute')
 
     class Meta:
         verbose_name = _('service provider options policy')
@@ -306,6 +318,80 @@ class SPOptionsIdPPolicy(models.Model):
     def __unicode__(self):
         return self.name
 
+class SAMLAttribute(models.Model):
+    ATTRIBUTE_NAME_FORMATS = (
+            ('basic', 'Basic'),
+            ('uri', 'URI'),
+    )
+    objects = a2_managers.GenericManager()
+
+    content_type = models.ForeignKey(ContentType,
+            verbose_name=_('content type'))
+    object_id = models.PositiveIntegerField(
+            verbose_name=_('object identifier'))
+    provider = GenericForeignKey('content_type', 'object_id')
+    name_format = models.CharField(
+            max_length=64,
+            verbose_name=_('name format'),
+            default='basic',
+            choices=ATTRIBUTE_NAME_FORMATS)
+    name = models.CharField(
+            max_length=64,
+            verbose_name=_('name'),
+            blank=True,
+            help_text=_('the local attribute name is used if left blank'))
+    friendly_name = models.CharField(
+            max_length=64,
+            verbose_name=_('friendly name'),
+            blank=True)
+    attribute_name = models.CharField(max_length=64,
+            verbose_name=_('attribute name'),
+            choices=(('', 'None'),))
+
+    def clean(self):
+        super(SAMLAttribute, self).clean()
+        if self.attribute_name and not self.name:
+            self.name = self.attribute_name
+
+    def name_format_uri(self):
+        if self.name_format == 'basic':
+            return lasso.SAML2_ATTRIBUTE_NAME_FORMAT_BASIC
+        elif self.name.format == 'uri':
+            return lasso.SAML2_ATTRIBUTE_NAME_FORMAT_URI
+        else:
+            raise NotImplementedError
+
+    def to_lasso_attribute(self, ctx):
+        if not self.attribute_name in ctx:
+            return None
+        at = lasso.Saml2Attribute()
+        at.nameFormat = self.name_format_uri()
+        at.name = self.name.encode('utf-8')
+        if self.friendly_name:
+            at.friendlyName = self.friendly_name.encode('utf-8')
+        values = ctx.get(self.attribute_name)
+        if isinstance(values, (basestring, numbers.Number, datetime.datetime, datetime.date)):
+            values = [values]
+        elif isinstance(values, (tuple, list)):
+            pass
+        else:
+            raise NotImplementedError
+        atv = lasso.Saml2AttributeValue()
+        atvs = []
+        for value in values:
+            value = value.encode('utf-8')
+            tn = lasso.MiscTextNode.newWithString(value)
+            tn.textChild = True
+            atv.any = [tn]
+            atvs.append(atv)
+        at.attributeValue = atvs
+        return at
+
+    def __unicode__(self):
+        ctx = {self.attribute_name: 'coin'}
+        return unicode(self.to_lasso_attribute(ctx).exportToXml(), 'utf-8')
+
+
 
 class AuthorizationAttributeMap(models.Model):
     name = models.CharField(max_length = 40, unique = True)
@@ -349,11 +435,6 @@ class AuthorizationSPPolicy(models.Model):
         return self.name
 
 
-class GetBySlugManager(models.Manager):
-    def get_by_natural_key(self, slug):
-        return self.get(slug=slug)
-
-
 class LibertyProvider(models.Model):
     name = models.CharField(max_length = 140,
             help_text = _("Internal nickname for the service provider"),
@@ -370,7 +451,7 @@ class LibertyProvider(models.Model):
     ca_cert_chain = models.TextField(blank=True)
     federation_source = models.CharField(max_length=64, blank=True, null=True)
 
-    objects = GetBySlugManager()
+    objects = managers.LibertyProviderManager()
 
     def __unicode__(self):
         return self.name
@@ -381,21 +462,6 @@ class LibertyProvider(models.Model):
             self.entity_id_sha1 = hashlib.sha1(self.entity_id).hexdigest()
         super(LibertyProvider, self).save(*args, **kwargs)
 
-    @classmethod
-    def get_provider_by_samlv2_artifact(cls, artifact):
-        '''Find a provider whose SHA-1 hash of its entityID is the 5-th to the
-           25-th byte of the given artifact'''
-        try:
-            artifact = base64.b64decode(artifact)
-        except:
-            raise ValueError('Artifact is not a base64 encoded value')
-        entity_id_sha1 = artifact[4:24]
-        entity_id_sha1 = binascii.hexlify(entity_id_sha1)
-        try:
-            return cls.objects.get(entity_id_sha1=entity_id_sha1)
-        except cls.DoesNotExist:
-            return None
-
     def clean(self):
         super(LibertyProvider, self).clean()
         p = lasso.Provider.newFromBuffer(lasso.PROVIDER_ROLE_ANY, self.metadata.encode('utf8'))
@@ -491,15 +557,6 @@ class LibertyIdentityProvider(models.Model):
         verbose_name = _('liberty identity provider')
         verbose_name_plural = _('liberty identity providers')
 
-class SessionLinkedManager(models.Manager):
-    def cleanup(self):
-        engine = import_module(settings.SESSION_ENGINE)
-        store = engine.SessionStore()
-        for o in self.all():
-            key = o.django_session_key
-            if not store.exists(key):
-                o.delete()
-
 LIBERTY_SESSION_DUMP_KIND_SP = 0
 LIBERTY_SESSION_DUMP_KIND_IDP = 1
 LIBERTY_SESSION_DUMP_KIND = { LIBERTY_SESSION_DUMP_KIND_SP: 'sp',
@@ -514,7 +571,7 @@ class LibertySessionDump(models.Model):
     session_dump = models.TextField(blank = True)
     kind = models.IntegerField(choices = LIBERTY_SESSION_DUMP_KIND.items())
 
-    objects = SessionLinkedManager()
+    objects = managers.SessionLinkedManager()
 
     class Meta:
         verbose_name = _('liberty session dump')
@@ -528,18 +585,12 @@ class LibertyManageDump(models.Model):
     django_session_key = models.CharField(max_length = 128)
     manage_dump = models.TextField(blank = True)
 
-    objects = SessionLinkedManager()
+    objects = managers.SessionLinkedManager()
 
     class Meta:
         verbose_name = _('liberty manage dump')
         verbose_name_plural = _('liberty manage dumps')
 
-class LibertyArtifactManager(models.Manager):
-    def cleanup(self):
-        expire = getattr(settings, 'SAML2_ARTIFACT_EXPIRATION', 600)
-        before = now()-datetime.timedelta(seconds=expire)
-        self.filter(creation__lt=before).delete()
-
 class LibertyArtifact(models.Model):
     """Store an artifact and the associated XML content"""
     creation = models.DateTimeField(auto_now_add=True)
@@ -547,7 +598,7 @@ class LibertyArtifact(models.Model):
     content = models.TextField()
     provider_id = models.CharField(max_length = 256)
 
-    objects = LibertyArtifactManager()
+    objects = managers.LibertyArtifactManager()
 
     class Meta:
         verbose_name = _('liberty artifact')
@@ -560,13 +611,6 @@ def nameid2kwargs(name_id):
         'name_id_content': name_id.content,
         'name_id_format': name_id.format }
 
-class LibertyAssertionManager(models.Manager):
-    def cleanup(self):
-        # keep assertions 1 week
-        expire = getattr(settings, 'SAML2_ASSERTION_EXPIRATION', 3600*24*7)
-        before = now()-datetime.timedelta(seconds=expire)
-        self.filter(creation__lt=before).delete()
-
 class LibertyAssertion(models.Model):
     assertion_id = models.CharField(max_length = 128)
     provider_id = models.CharField(max_length = 256)
@@ -587,18 +631,8 @@ class LibertyAssertion(models.Model):
         verbose_name = _('liberty assertion')
         verbose_name_plural = _('liberty assertions')
 
-
-federation_delete = Signal()
-
-class LibertyFederationManager(models.Manager):
-    def cleanup(self):
-        for federation in self.filter(user__isnull=True):
-            results = federation_delete.send_robust(sender=federation)
-            for callback, result in results:
-                if not result:
-                    return
-            federation.delete()
-
+# XXX: for retrocompatibility
+federation_delete = managers.federation_delete
 
 class LibertyFederation(models.Model):
     """Store a federation, i.e. an identifier shared with another provider, be
@@ -619,7 +653,7 @@ class LibertyFederation(models.Model):
     creation = models.DateTimeField(auto_now_add=True)
     last_modification = models.DateTimeField(auto_now=True)
 
-    objects = LibertyFederationManager()
+    objects = managers.LibertyFederationManager()
 
     def __init__(self, *args, **kwargs):
         saml2_assertion = kwargs.pop('saml2_assertion', None)
@@ -653,8 +687,10 @@ class LibertySession(models.Model):
     django_session_key = models.CharField(max_length = 128)
     session_index = models.CharField(max_length = 80)
     provider_id = models.CharField(max_length = 256)
-    federation = models.ForeignKey(LibertyFederation, null = True)
-    assertion = models.ForeignKey(LibertyAssertion, null = True)
+    federation = models.ForeignKey(LibertyFederation, blank=True,
+            null = True)
+    assertion = models.ForeignKey(LibertyAssertion, blank=True, null
+            = True)
     name_id_qualifier = models.CharField(max_length = 256,
             verbose_name = _("Qualifier"), null = True)
     name_id_format = models.CharField(max_length = 100,
@@ -665,7 +701,7 @@ class LibertySession(models.Model):
             verbose_name = _("SPNameQualifier"), null = True)
     creation = models.DateTimeField(auto_now_add=True)
 
-    objects = SessionLinkedManager()
+    objects = managers.LibertySessionManager()
 
     def __init__(self, *args, **kwargs):
         saml2_assertion = kwargs.pop('saml2_assertion', None)
@@ -680,10 +716,19 @@ class LibertySession(models.Model):
         self.__dict__.update(nameid2kwargs(name_id))
 
     @classmethod
-    def get_for_nameid_and_session_indexes(cls, name_id, session_indexes):
+    def get_for_nameid_and_session_indexes(cls, issuer_id, provider_id, name_id, session_indexes):
         kwargs = nameid2kwargs(name_id)
-        return LibertySession.objects.filter(session_index__in=session_indexes,
-                **kwargs)
+        name_id_qualifier = kwargs['name_id_qualifier']
+        qs = LibertySession.objects.filter(provider_id=provider_id,
+                session_index__in=session_indexes)
+        if name_id_qualifier and name_id_qualifier != issuer_id:
+            qs = qs.filter(**kwargs)
+        else:
+            kwargs.pop('name_id_qualifier')
+            qs = qs.filter(**kwargs) \
+                    .filter(Q(name_id_qualifier__isnull=True)|Q(name_id_qualifier=issuer_id))
+        qs = qs.filter(Q(name_id_sp_name_qualifier__isnull=True)|Q(name_id_sp_name_qualifier=provider_id))
+        return qs
 
     def __unicode__(self):
         return '<LibertySession %s>' % self.__dict__
